@@ -65,7 +65,7 @@ class imageDisk {
 		virtual void Get_Geometry(uint32_t * getHeads, uint32_t *getCyl, uint32_t *getSect, uint32_t *getSectSize);
 		virtual uint8_t GetBiosType(void);
 		virtual uint32_t getSectSize(void);
-		imageDisk(class DOS_Drive *useDrive, unsigned int letter, int freeMB, int timeout);
+		imageDisk(class DOS_Drive *useDrive, unsigned int letter, uint32_t freeMB, int timeout);
 		imageDisk(FILE *imgFile, const char *imgName, uint32_t imgSizeK, bool isHardDisk);
 		imageDisk(FILE* diskimg, const char* diskName, uint32_t cylinders, uint32_t heads, uint32_t sectors, uint32_t sector_size, bool hardDrive);
 		virtual ~imageDisk();
@@ -95,7 +95,7 @@ class imageDisk {
 	private:
 		volatile int refcount = 0;
 		std::vector<bool> partition_in_use; /* used by FAT driver to prevent mounting a partition twice */
-		uint64_t current_fpos;
+		uint64_t current_fpos = 0;
 
 	public:
 		int Addref() {
@@ -300,6 +300,7 @@ public:
 	virtual uint8_t Write_AbsoluteSector(uint32_t sectnum, const void * data);
 	static ErrorCodes Open(const char* fileName, const bool readOnly, imageDisk** disk);
 	static VHDTypes GetVHDType(const char* fileName);
+	VHDTypes GetVHDType(void) const;
 	virtual ~imageDiskVHD();
 
 private:
