@@ -234,7 +234,7 @@ bool DOS_CreateTempFile(char * const name,uint16_t * entry);
 bool DOS_FileExists(char const * const name);
 
 /* Helper Functions */
-bool DOS_MakeName(char const * const name,char * const fullname,uint8_t * drive);
+bool DOS_MakeName(char const * const name,char * const fullname,uint8_t * drive,bool isVolume = false);
 /* Drive Handing Routines */
 uint8_t DOS_GetDefaultDrive(void);
 void DOS_SetDefaultDrive(uint8_t drive);
@@ -539,6 +539,9 @@ public:
 	RealPt	GetPointer(void);
 	uint32_t GetDeviceChain(void);
 
+	void SetBootDrive(uint8_t drv) { sSave(sDIB,bootDrive,drv); }
+	uint8_t GetBootDrive(void) { return sGet(sDIB,bootDrive); }
+
 	#ifdef _MSC_VER
 	#pragma pack(1)
 	#endif
@@ -1025,4 +1028,6 @@ const std::map<std::string, int> country_code_map {
 	{"yu",     COUNTRYNO::Yugoslavia     },
 };
 
+void DOS_FlushSTDIN(void);
+
 #endif
