@@ -1,7 +1,7 @@
 
 #include "dos_inc.h"
 
-static const unsigned char bin_mem_com[] = {
+static const unsigned char bin_mem_exe[] = {
 0x4d,0x5a,0x21,0x01,0x1e,0x00,0x01,0x00,0x02,0x00,0xbd,0x09,0xff,0xff,0xed,0x0c,
 0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x1c,0x00,0x00,0x00,0x0a,0x00,0xaf,0x03,
 0xb9,0x56,0x1d,0xbe,0xaa,0x3a,0x89,0xf7,0x1e,0xa9,0xb5,0x80,0x8c,0xc8,0x05,0x05,
@@ -950,9 +950,9 @@ static const unsigned char bin_mem_com[] = {
 0xf3,0x07,0x06,0x1f,0xbc,0x00,0x08,0xea,0x62,0x35,0x00,0x00,0x00,0x08,0xda,0x07,
 0x1a};
 
-struct BuiltinFileBlob bfb_MEM_COM = {
-	/*recommended file name*/	"MEM.COM",
-	/*data*/			bin_mem_com,
-	/*length*/			sizeof(bin_mem_com)
+struct BuiltinFileBlob bfb_MEM_EXE = {
+	/*recommended file name*/	"MEM.EXE",
+	/*data*/			bin_mem_exe,
+	/*length*/			sizeof(bin_mem_exe)
 };
 
