@@ -27,6 +27,7 @@
 
 #include "logging.h"
 #include "support.h"
+#include "drives.h"
 #include "cdrom.h"
 
 #if defined(C_SDL2)
@@ -216,13 +217,6 @@ int CDROM_GetMountType(const char* path, int forceCD) {
 	// Detect ISO
     struct pref_stat file_stat;
 #if defined(WIN32)
-# if defined(__MINGW32__)
-#  define ht_stat_t struct _stat
-#  define ht_stat(x,y) _wstat(x,y)
-# else
-#  define ht_stat_t struct _stat64
-#  define ht_stat(x,y) _wstat64(x,y)
-# endif
     ht_stat_t hfile_stat;
     typedef wchar_t host_cnv_char_t;
     host_cnv_char_t *CodePageGuestToHost(const char *s);
