@@ -347,6 +347,7 @@ public:
 	bool	ReadSector              (uint8_t *buffer, bool raw, unsigned long sector);
 	//! \brief Indicate whether the image has a data track
 	bool	HasDataTrack            (void);
+	bool	HasAudioTrack           (void);
     //! \brief Flag to track if images have been initialized
     //!
     //! \description Whether images[] has been initialized.
@@ -461,7 +462,7 @@ private:
 	HANDLE		hEvent;											// global event
 	DWORD		(*pGetASPI32SupportInfo)	(void);             // ptrs to aspi funcs
 	DWORD		(*pSendASPI32Command)		(LPSRB);
-	TMSF		oldLeadOut;
+    TMSF        oldLeadOut = {};
 };
 
 class CDROM_Interface_Ioctl : public CDROM_Interface
