@@ -108,7 +108,7 @@ bool DOS_ExtDevice::Read(uint8_t * data,uint16_t * size) {
 	PhysPt bufptr = (dos.dcp << 4) | 32;
 	for(uint16_t no = 0 ; no < *size ; no++) {
 		// INPUT
-		if((CallDeviceFunction(4, 26, dos.dcp + 2, 0, 1) & 0x8000)) {
+		if(CallDeviceFunction(4, 26, dos.dcp + 2, 0, 1) & 0x8000) {
 			return false;
 		} else {
 			if(real_readw(dos.dcp, 18) != 1) {
@@ -125,7 +125,7 @@ bool DOS_ExtDevice::Write(const uint8_t * data,uint16_t * size) {
 	for(uint16_t no = 0 ; no < *size ; no++) {
 		mem_writeb(bufptr, *data);
 		// OUTPUT
-		if((CallDeviceFunction(8, 26, dos.dcp + 2, 0, 1) & 0x8000)) {
+		if(CallDeviceFunction(8, 26, dos.dcp + 2, 0, 1) & 0x8000) {
 			return false;
 		} else {
 			if(real_readw(dos.dcp, 18) != 1) {
@@ -465,7 +465,7 @@ private:
 			if (EmptyClipboard())
 				{
 				int bytes = ftell(fh);
-				HGLOBAL hCbData = GlobalAlloc(NULL, bytes);
+				HGLOBAL hCbData = GlobalAlloc(0, bytes);
 				uint8_t* pChData = (uint8_t*)GlobalLock(hCbData);
 				if (pChData)
 					{
@@ -641,6 +641,12 @@ uint16_t DOS_Device::GetInformation(void) {
 	return Devices[devnum]->GetInformation();
 }
 
+void DOS_Device::SetInformation(uint16_t info) {
+	if(Devices[devnum]->IsName("CON") && !(Devices[devnum]->GetInformation() & EXT_DEVICE_BIT)) {
+		Devices[devnum]->SetInformation(info);
+	}
+}
+
 bool DOS_Device::ReadFromControlChannel(PhysPt bufptr,uint16_t size,uint16_t * retcode) {
 	return Devices[devnum]->ReadFromControlChannel(bufptr,size,retcode);
 }
@@ -690,9 +696,26 @@ uint8_t DOS_FindDevice(char const * name) {
 	char fullname[DOS_PATHLENGTH];uint8_t drive;
 	bool ime_flag = false;
 //	if(!name || !(*name)) return DOS_DEVICES; //important, but makename does it
+
+	// NTS: If a program is trying to open paths like "@:\\WINBOOT.SYS" the most likely
+	//      reason is that it's a Microsoft program like FORMAT.COM and it's reading
+	//      locations in the DOS kernel that are supposed to hold the drive letter of
+	//      the boot drive, but carry zero instead.
+	//
+	//      "@:" paths are a sign the program is possibly trying to read files from
+	//      the boot device but is using nonstandard means to determine that i.e.
+	//      not using INT 21h AX=3305h as documented for MS-DOS 4.0 or higher.
+	//      Real MS-DOS probably does not accept "@:" drive paths and neither should
+	//      we.
+	//
+	//      The other reason for this rejection is that the code that was here to
+	//      convert "@:" to normal paths did not initialize local variable "drive"
+	//      and Windows 95 FORMAT.COM requesting "@:\\WINBOOT.SYS" prior to this fix
+	//      sometimes caused a segfault.
 	if(*name == '@' && *(name + 1) == ':') {
-		strcpy(fullname, name + 2);
-		ime_flag = true;
+		LOG_MSG("DOS_FindDevice(): Rejecting path '%s'. @: paths are not valid. It may be a sign the program is attempting to locate the boot drive in an undocumented manner not supported by this emulator",name);
+		return DOS_DEVICES;
+		//ime_flag = true;   // FIXME: <- Why?
 	} else {
 		if (!DOS_MakeName(name,fullname,&drive)) return DOS_DEVICES;
 	}
