@@ -1015,7 +1015,6 @@ isoDrive::isoDrive(char driveLetter, const char* fileName, uint8_t mediaid, int&
 	this->discLabel[0] = '\0';
 	subUnit = 0;
 	nextFreeDirIterator = 0;
-	memset(dirIterators, 0, sizeof(dirIterators));
 	memset(sectorHashEntries, 0, sizeof(sectorHashEntries));
 	memset(&rootEntry, 0, sizeof(isoDirEntry));
 	
@@ -1032,7 +1031,7 @@ isoDrive::isoDrive(char driveLetter, const char* fileName, uint8_t mediaid, int&
 			if (!MSCDEX_GetVolumeName(subUnit, buffer)) strcpy(buffer, "");
 			Set_Label(buffer,discLabel,true);
 
-		} else if (CDROM_Interface_Image::images[subUnit]->HasDataTrack() == false) { //Audio only cdrom
+		} else if (CDROM_Interface_Image::images[subUnit]->HasDataTrack() == false && CDROM_Interface_Image::images[subUnit]->HasAudioTrack() == true) { //Audio only cdrom
 			strcpy(info, "isoDrive ");
 			strcat(info, fileName);
 			this->driveLetter = driveLetter;
@@ -1430,6 +1429,7 @@ Bits isoDrive::UnMount(void) {
 }
 
 int isoDrive::GetDirIterator(const UDFFileEntry &fe) {
+    (void)fe;
 	if (!is_udf) return 0;
 
 	int dirIterator = nextFreeDirIterator;
@@ -2325,16 +2325,26 @@ void isoDrive :: MediaChange() {
 }
 
 void isoDrive :: EmptyCache(void) {
+	// Magical Girl Pretty Sammy installer
+	// Installer copies files found by FindFirst/FindNext with "command /c copy",
+	// this function is called at end of DOS_Shell::CMD_COPY and cache is cleared, so only one file is copied.
+	if(IS_PC98_ARCH) {
+		const char *label = GetLabel();
+		if(!strncmp(label, "SAMY_A98", 8) || !strncmp(label, "SAMY_B98", 8)) {
+			// Do not clear cache for Pretty Sammy CD-ROM
+			return;
+		}
+	}
 	enable_udf = (dos.version.major > 7 || (dos.version.major == 7 && dos.version.minor >= 10));//default
 	enable_rock_ridge = dos.version.major >= 7 || uselfn;
 	enable_joliet = dos.version.major >= 7 || uselfn;
 	is_joliet = false;
-	this->fileName[0]  = '\0';
-	this->discLabel[0] = '\0';
+	//this->fileName[0]  = '\0'; /* deleted to fix issue #3848. Revert this if there are any flaws */
+	//this->discLabel[0] = '\0'; /* deleted to fix issue #3848. Revert this if there are any flaws */
 	nextFreeDirIterator = 0;
 	memset(dirIterators, 0, sizeof(dirIterators));
 	memset(sectorHashEntries, 0, sizeof(sectorHashEntries));
 	memset(&rootEntry, 0, sizeof(isoDirEntry));
-	safe_strncpy(this->fileName, fileName, CROSS_LEN);
+	//safe_strncpy(this->fileName, fileName, CROSS_LEN); /* deleted to fix issue #3848. Revert this if there are any flaws */
 	loadImage();
 }
