@@ -260,7 +260,14 @@ int get_expanded_files(const std::string &path, std::vector<std::string> &paths,
     if (d) {
         while ((dir = readdir(d)) != NULL) {
             host_cnv_char_t *temp_name = CodePageHostToGuest(dir->d_name);
-            if (dir->d_type == DT_REG)
+#if defined(HAIKU)
+            struct stat path_stat;
+            stat(dir->d_name, &path_stat);
+            bool is_regular_file = S_ISREG(path_stat.st_mode);
+#else
+            bool is_regular_file = (dir->d_type == DT_REG);
+#endif
+            if (is_regular_file)
                 names.push_back(temp_name!=NULL?temp_name:dir->d_name);
         }
         closedir(d);
@@ -452,22 +459,29 @@ void DriveManager::CycleDisk(bool pressed) {
 }
 */
 
-void DriveManager::CycleDisks(int drive, bool notify, int position) {
+void DriveManager::CycleDisks(int drive, bool notify, unsigned int position) {
 	unsigned int numDisks = (unsigned int)driveInfos[drive].disks.size();
 	if (numDisks > 1) {
 		// cycle disk
-		unsigned int currentDisk = (unsigned int)driveInfos[drive].currentDisk;
-        const DOS_Drive* oldDisk = driveInfos[drive].disks[(unsigned int)currentDisk];
+		unsigned int currentDisk = driveInfos[drive].currentDisk;
+        const DOS_Drive* oldDisk = driveInfos[drive].disks[currentDisk];
         if (position<1)
-            currentDisk = ((unsigned int)currentDisk + 1u) % (unsigned int)numDisks;
+            currentDisk = (currentDisk + 1u) % numDisks;
         else if (position>numDisks)
             currentDisk = 0;
         else
             currentDisk = position - 1;
 		DOS_Drive* newDisk = driveInfos[drive].disks[currentDisk];
 		driveInfos[drive].currentDisk = currentDisk;
+		if (drive < MAX_DISK_IMAGES && imageDiskList[drive] != NULL) {
+			if (strncmp(newDisk->GetInfo(),"fatDrive",8) == 0)
+				imageDiskList[drive] = ((fatDrive *)newDisk)->loadedDisk;
+			else
+				imageDiskList[drive] = (imageDisk *)newDisk;
+			if ((drive == 2 || drive == 3) && imageDiskList[drive]->hardDrive) updateDPT();
+		}
 		
-		// copy working directory, acquire system resources and finally switch to next drive		
+		// copy working directory, acquire system resources and finally switch to next drive
 		strcpy(newDisk->curdir, oldDisk->curdir);
 		newDisk->Activate();
         if (!dos_kernel_disabled) newDisk->UpdateDPB(currentDrive);
@@ -488,11 +502,11 @@ void DriveManager::CycleAllCDs(void) {
 			unsigned int currentDisk = driveInfos[idrive].currentDisk;
             const DOS_Drive* oldDisk = driveInfos[idrive].disks[currentDisk];
             if (dynamic_cast<const isoDrive*>(oldDisk) == NULL) continue;
-			currentDisk = ((unsigned int)currentDisk + 1u) % (unsigned int)numDisks;		
+			currentDisk = (currentDisk + 1u) % numDisks;
 			DOS_Drive* newDisk = driveInfos[idrive].disks[currentDisk];
 			driveInfos[idrive].currentDisk = currentDisk;
 			
-			// copy working directory, acquire system resources and finally switch to next drive		
+			// copy working directory, acquire system resources and finally switch to next drive
 			strcpy(newDisk->curdir, oldDisk->curdir);
 			newDisk->Activate();
             if (!dos_kernel_disabled) newDisk->UpdateDPB(currentDrive);
