@@ -43,7 +43,7 @@ public:
 	static char * GetDrivePosition(int drive);
 //	static void CycleDrive(bool pressed);
 //	static void CycleDisk(bool pressed);
-	static void CycleDisks(int drive, bool notify, int position=0);
+	static void CycleDisks(int drive, bool notify, unsigned int position=0);
 	static void CycleAllDisks(void);
 	static void CycleAllCDs(void);
 	static void Init(Section* s);
@@ -398,6 +398,7 @@ public:
 	uint8_t readSector(uint32_t sectnum, void * data);
 	uint8_t writeSector(uint32_t sectnum, void * data);
 	uint32_t getAbsoluteSectFromBytePos(uint32_t startClustNum, uint32_t bytePos);
+	uint32_t getSectorCount(void);
 	uint32_t getSectorSize(void);
 	uint32_t getClusterSize(void);
 	uint32_t getAbsoluteSectFromChain(uint32_t startClustNum, uint32_t logicalSector);
@@ -412,6 +413,7 @@ public:
 	imageDisk *loadedDisk = NULL;
 	uint8_t req_ver_major = 0,req_ver_minor = 0;
 	bool created_successfully = true;
+	uint32_t partSectOff;
     struct {
         uint32_t bytesector;
         uint32_t cylsector;
@@ -472,11 +474,11 @@ private:
 	bool absolute = false;
 	uint8_t fattype = 0;
 	uint32_t CountOfClusters = 0;
-	uint32_t partSectOff = 0;
 	uint32_t partSectSize = 0;
 	uint32_t firstDataSector = 0;
 	uint32_t firstRootDirSect = 0;
 	uint32_t physToLogAdj = 0; // Some PC-98 HDI images have larger logical than physical bytes/sector and the partition is not a multiple of it, so this is needed
+	uint32_t searchFreeCluster = 0;
 	int partition_index = -1;
 
 	uint32_t cwdDirCluster = 0;
@@ -1126,7 +1128,7 @@ private:
 		uint32_t index;
 		uint32_t pos;
 		UDFextents udfdirext;
-	} dirIterators[MAX_OPENDIRS];
+    } dirIterators[MAX_OPENDIRS] = {};
 private:
 	bool GetNextDirEntry(const int dirIteratorHandle, UDFFileIdentifierDescriptor &fid, UDFFileEntry &fe, UDFextents &dirext, char fname[LFN_NAMELENGTH],unsigned int dirIteratorIndex);
 };
