@@ -617,7 +617,7 @@ void FPU_ESC6_Normal(Bitu rm) {
 		break;
 	case 0x02:  /* FCOMP5*/
 		FPU_FCOM(TOP,STV(sub));
-		break;	/* TODO IS THIS ALLRIGHT ????????? */
+		break;	/* TODO IS THIS ALRIGHT ????????? */
 	case 0x03:  /*FCOMPP*/
 		if(sub != 1) {
 			LOG(LOG_FPU,LOG_WARN)("ESC 6:Unhandled group %d subfunction %d",(int)group,(int)sub);
@@ -1196,4 +1196,4 @@ std::string FPUStatusWord::to_string() const
 	       " UE=" + std::to_string(UE) + " OE=" + std::to_string(OE) +
 	       " ZE=" + std::to_string(ZE) + " DE=" + std::to_string(DE) +
 	       " IE=" + std::to_string(IE) + " TOP=" + std::to_string(top);
-}
\ No newline at end of file
+}
