@@ -31,12 +31,16 @@
 #include "control.h"
 #include "mapper.h"
 #include "menudef.h"
+#include "vga.h"
 #include "pic.h"
 #include "cross.h"
 #include "hardware.h"
 #include "support.h"
 #include "sdlmain.h"
 #include "shell.h"
+#include "pc98_cg.h"
+#include "pc98_gdc.h"
+#include "pc98_gdc_const.h"
 
 #include "render_scalers.h"
 #include "render_glsl.h"
@@ -45,6 +49,8 @@
 #include <emmintrin.h>
 #endif
 
+extern bool video_debug_overlay;
+
 Render_t                                render;
 int                                     eurAscii = -1;
 int                                     aspect_ratio_x = 0;
@@ -379,10 +385,15 @@ void AspectRatio_mapper_shortcut(bool pressed) {
     }
 }
 
+void VGA_DebugOverlay();
+
 void RENDER_EndUpdate( bool abort ) {
     if (GCC_UNLIKELY(!render.updating))
         return;
 
+    if (video_debug_overlay && !abort && render.active)
+        VGA_DebugOverlay();
+
     if (!abort && render.active && RENDER_DrawLine == RENDER_ClearCacheHandler)
         render.scale.clearCache = false;
 
@@ -887,6 +898,69 @@ void RENDER_SetSize(Bitu width,Bitu height,Bitu bpp,float fps,double scrn_ratio)
     }
     LOG_MSG("pixratio %1.3f, dw %s, dh %s",ratio,dblw?"true":"false",dblh?"true":"false");
 
+    /* this must be done after dblw/dblh so extra room can be added without screwing up the screen.
+     * debug information is drawn into the buffer pre-scaler to make sure that if the user wants it
+     * in still image or video capture, they can. */
+    if (video_debug_overlay) {
+	if (width < 320) width = 320;
+	height += 4;
+	if (machine == MCH_EGA) {
+		height += 8*3;
+		width += 4;
+		width += 16*2; /* palette */
+		width += 4;
+		width += 4*2; /* cpe */
+		width += 4;
+		width += 8*32;
+		width += 4;
+	}
+	else if (machine == MCH_MDA || machine == MCH_HERC) {
+		/* add nothing, nothing to show at this time */
+	}
+	else if (machine == MCH_PC98) {
+		height += 8*6;
+		width += 4;
+		if (pc98_gdc_vramop & (1u << VOPBIT_VGA)) {
+			/* PC-98 games do not often use the 256-color mode, and if they do,
+			 * they do not do "copper" effects or per-scanline effects. */
+		}
+		else if (pc98_gdc_vramop & (1u << VOPBIT_ANALOG)) {
+			/* 16-color "analog". Though uncommon, there is at least one test
+			 * case I am aware of that does a per-scanline "copper" effect
+			 * in the title screen though it's more of a momentary flash of
+			 * light with a vertical gradient. */
+			width += 16*2;
+			width += 4;
+		}
+		else {
+			/* 8-color "digital". I can't think of any reason you'd want to
+			 * do "copper" effects when all you can do is map one 3-bit value
+			 * to another 3-bit value. It's like, what's the point? Woo. Well,
+			 * in case some game has a reason, show it. */
+			width += 8*2;
+			width += 4;
+		}
+		width += 8*32;
+		width += 4;
+	}
+	else if (machine == MCH_VGA) {
+		height += 8*7;
+		width += 4;
+		if (vga.mode == M_VGA || vga.mode == M_LIN8) {
+			width += 256;
+			width += 4;
+		}
+		width += 16; /* CSPAL */
+		width += 4;
+		width += 8*32;
+		width += 4;
+	}
+	else {
+		height += 8*2;
+	}
+	height += 4;
+    }
+
     if ( ratio > 1.0 ) {
         double target = height * ratio + 0.025;
         ratio = target / height;
@@ -1329,7 +1403,7 @@ void RENDER_Init() {
     render.forceUpdate=false;
     std::string cline;
     std::string scaler;
-    //Check for commandline paramters and parse them through the configclass so they get checked against allowed values
+    //Check for commandline parameters and parse them through the configclass so they get checked against allowed values
     if (control->cmdline->FindString("-scaler",cline,true)) {
         section->HandleInputline(std::string("scaler=") + cline);
     } else if (control->cmdline->FindString("-forcescaler",cline,true)) {
@@ -1338,9 +1412,6 @@ void RENDER_Init() {
 
     RENDER_UpdateFromScalerSetting();
 
-    vga_alt_new_mode = control->opt_alt_vga_render || section->Get_bool("alt render");
-    if (vga_alt_new_mode) LOG_MSG("Alternative VGA render engine not yet fully implemented!");
-
     render.autofit=section->Get_bool("autofit");
 
 #if C_OPENGL
