@@ -52,19 +52,21 @@
 #if (C_AVCODEC)
 extern "C" {
 #include <libavutil/pixfmt.h>
+#include <libavutil/channel_layout.h>
 #include <libswscale/swscale.h>
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libavformat/avio.h>
 }
 
-/* This code now requires FFMPEG 4.0.2 or higher */
-# if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58,18,100)
+/* This code now requires FFMPEG 4 or higher */
+# if LIBAVCODEC_VERSION_MAJOR < 58
 #  error Your libavcodec is too old. Update FFMPEG.
 # endif
 
 #endif
 
+bool video_debug_overlay = false;
 bool skip_encoding_unchanged_frames = false, show_recorded_filename = true;
 std::string pathvid = "", pathwav = "", pathmtw = "", pathmid = "", pathopl = "", pathscr = "", pathprt = "";
 bool systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
@@ -72,9 +74,9 @@ bool systemmessagebox(char const * aTitle, char const * aMessage, char const * a
 #if (C_AVCODEC)
 bool ffmpeg_init = false;
 AVFormatContext*	ffmpeg_fmt_ctx = NULL;
-AVCodec*		ffmpeg_vid_codec = NULL;
+const AVCodec*		ffmpeg_vid_codec = NULL;
 AVCodecContext*		ffmpeg_vid_ctx = NULL;
-AVCodec*		ffmpeg_aud_codec = NULL;
+const AVCodec*		ffmpeg_aud_codec = NULL;
 AVCodecContext*		ffmpeg_aud_ctx = NULL;
 AVStream*		ffmpeg_vid_stream = NULL;
 AVStream*		ffmpeg_aud_stream = NULL;
@@ -111,15 +113,21 @@ void ffmpeg_closeall() {
 			ffmpeg_avformat_began = false;
 		}
 		avio_close(ffmpeg_fmt_ctx->pb);
+		if (ffmpeg_vid_ctx != NULL) avcodec_close(ffmpeg_vid_ctx);
+		if (ffmpeg_aud_ctx != NULL) avcodec_close(ffmpeg_aud_ctx);
 		avformat_free_context(ffmpeg_fmt_ctx);
 		ffmpeg_fmt_ctx = NULL;
+		ffmpeg_vid_ctx = NULL; // NTS: avformat_free_context() freed this for us, don't free again
+		ffmpeg_aud_ctx = NULL; // NTS: avformat_free_context() freed this for us, don't free again
 	}
 	if (ffmpeg_vid_ctx != NULL) {
 		avcodec_close(ffmpeg_vid_ctx);
+		avcodec_free_context(&ffmpeg_vid_ctx);
 		ffmpeg_vid_ctx = NULL;
 	}
 	if (ffmpeg_aud_ctx != NULL) {
 		avcodec_close(ffmpeg_aud_ctx);
+		avcodec_free_context(&ffmpeg_aud_ctx);
 		ffmpeg_aud_ctx = NULL;
 	}
 	if (ffmpeg_vidrgb_frame != NULL) {
@@ -148,34 +156,30 @@ void ffmpeg_closeall() {
 }
 
 void ffmpeg_audio_frame_send() {
-	AVPacket pkt;
-	int gotit;
+	AVPacket* pkt = av_packet_alloc();
+	int r;
 
-	av_init_packet(&pkt);
-	if (av_new_packet(&pkt,65536) == 0) {
-		pkt.flags |= AV_PKT_FLAG_KEY;
+	if (!pkt) E_Exit("Error: Unable to alloc packet");
 
-		if (avcodec_encode_audio2(ffmpeg_aud_ctx,&pkt,ffmpeg_aud_frame,&gotit) == 0) {
-			if (gotit) {
-				pkt.pts = (int64_t)ffmpeg_audio_sample_counter;
-				pkt.dts = (int64_t)ffmpeg_audio_sample_counter;
-				pkt.stream_index = ffmpeg_aud_stream->index;
-				av_packet_rescale_ts(&pkt,ffmpeg_aud_ctx->time_base,ffmpeg_aud_stream->time_base);
+	ffmpeg_aud_frame->key_frame = 1;
+	ffmpeg_aud_frame->pts = (int64_t)ffmpeg_audio_sample_counter;
+	r=avcodec_send_frame(ffmpeg_aud_ctx,ffmpeg_aud_frame);
+	if (r < 0 && r != AVERROR(EAGAIN))
+		LOG_MSG("WARNING: avcodec_send_frame() audio failed to encode (err=%d)",r);
 
-				if (av_interleaved_write_frame(ffmpeg_fmt_ctx,&pkt) < 0)
-					LOG_MSG("WARNING: av_interleaved_write_frame failed");
-			}
-			else {
-				LOG_MSG("DEBUG: avcodec_encode_audio2() delayed output");
-			}
-		}
-		else {
-			LOG_MSG("WARNING: avcodec_encode_audio2() failed to encode");
-		}
+	while ((r=avcodec_receive_packet(ffmpeg_aud_ctx,pkt)) >= 0) {
+		pkt->stream_index = ffmpeg_aud_stream->index;
+		av_packet_rescale_ts(pkt,ffmpeg_aud_ctx->time_base,ffmpeg_aud_stream->time_base);
+
+		if (av_interleaved_write_frame(ffmpeg_fmt_ctx,pkt) < 0)
+			LOG_MSG("WARNING: av_interleaved_write_frame failed");
 	}
-	av_packet_unref(&pkt);
+
+	if (r != AVERROR(EAGAIN))
+		LOG_MSG("WARNING: avcodec_receive_packet() audio failed to encode (err=%d)",r);
 
 	ffmpeg_audio_sample_counter += (uint64_t)ffmpeg_aud_frame->nb_samples;
+	av_packet_free(&pkt);
 }
 
 void ffmpeg_take_audio(int16_t *raw,unsigned int samples) {
@@ -238,78 +242,82 @@ void ffmpeg_take_audio(int16_t *raw,unsigned int samples) {
 void ffmpeg_reopen_video(double fps);
 
 void ffmpeg_flush_video() {
+	/* video */
 	if (ffmpeg_fmt_ctx != NULL) {
 		if (ffmpeg_vid_frame != NULL) {
-			bool again;
-			AVPacket pkt;
+			signed long long saved_dts;
+			AVPacket* pkt = av_packet_alloc();
+			int r;
 
-			do {
-				again=false;
-				av_init_packet(&pkt);
-				if (av_new_packet(&pkt,50000000/8) == 0) {
-					int gotit=0;
-					if (avcodec_encode_video2(ffmpeg_vid_ctx,&pkt,NULL,&gotit) == 0) {
-						if (gotit) {
-							uint64_t tm;
+			if (!pkt) E_Exit("Error: Unable to alloc packet");
 
-							again = true;
-							tm = (uint64_t)pkt.pts;
-							pkt.stream_index = ffmpeg_vid_stream->index;
-							av_packet_rescale_ts(&pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
-							pkt.pts += (int64_t)ffmpeg_video_frame_time_offset;
-							pkt.dts += (int64_t)ffmpeg_video_frame_time_offset;
+			r=avcodec_send_frame(ffmpeg_vid_ctx,NULL);
+			if (r < 0 && r != AVERROR(EAGAIN))
+				LOG_MSG("WARNING: avcodec_send_frame() video flush failed to encode (err=%d)",r);
 
-							if (av_interleaved_write_frame(ffmpeg_fmt_ctx,&pkt) < 0)
-								LOG_MSG("WARNING: av_interleaved_write_frame failed");
+			while ((r=avcodec_receive_packet(ffmpeg_vid_ctx,pkt)) >= 0) {
+				saved_dts = pkt->dts;
+				pkt->stream_index = ffmpeg_vid_stream->index;
+				av_packet_rescale_ts(pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
+				pkt->pts += (int64_t)ffmpeg_video_frame_time_offset;
+				pkt->dts += (int64_t)ffmpeg_video_frame_time_offset;
 
-							pkt.pts = (int64_t)tm + (int64_t)1;
-							pkt.dts = (int64_t)tm + (int64_t)1;
-							av_packet_rescale_ts(&pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
-							ffmpeg_video_frame_last_time = (uint64_t)pkt.pts;
-						}
-					}
-				}
-				av_packet_unref(&pkt);
-			} while (again);
+				if (av_interleaved_write_frame(ffmpeg_fmt_ctx,pkt) < 0)
+					LOG_MSG("WARNING: av_interleaved_write_frame failed");
+
+				pkt->pts = (int64_t)saved_dts + (int64_t)1;
+				pkt->dts = (int64_t)saved_dts + (int64_t)1;
+				av_packet_rescale_ts(pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
+				ffmpeg_video_frame_last_time = (uint64_t)pkt->pts;
+			}
+
+			if (r != AVERROR(EAGAIN) && r != AVERROR_EOF)
+				LOG_MSG("WARNING: avcodec_receive_packet() video flush failed to encode (err=%d)",r);
+
+			av_packet_free(&pkt);
+		}
+	}
+}
+
+void ffmpeg_flush_audio() {
+	/* audio */
+	if (ffmpeg_fmt_ctx != NULL) {
+		if (ffmpeg_aud_frame != NULL) {
+			AVPacket* pkt = av_packet_alloc();
+			int r;
+
+			if (!pkt) E_Exit("Error: Unable to alloc packet");
+
+			if (ffmpeg_aud_write != 0) {
+				ffmpeg_aud_frame->nb_samples = (int)ffmpeg_aud_write;
+				ffmpeg_audio_frame_send();
+			}
+
+			ffmpeg_aud_write = 0;
+			ffmpeg_aud_frame->nb_samples = 0;
+
+			r=avcodec_send_frame(ffmpeg_aud_ctx,NULL);
+			if (r < 0 && r != AVERROR(EAGAIN))
+				LOG_MSG("WARNING: avcodec_send_frame() audio flush failed to encode (err=%d)",r);
+
+			while ((r=avcodec_receive_packet(ffmpeg_aud_ctx,pkt)) >= 0) {
+				pkt->stream_index = ffmpeg_aud_stream->index;
+				av_packet_rescale_ts(pkt,ffmpeg_aud_ctx->time_base,ffmpeg_aud_stream->time_base);
+
+				if (av_interleaved_write_frame(ffmpeg_fmt_ctx,pkt) < 0)
+					LOG_MSG("WARNING: av_interleaved_write_frame failed");
+			}
+
+			if (r != AVERROR(EAGAIN) && r != AVERROR_EOF)
+				LOG_MSG("WARNING: avcodec_receive_packet() audio flush failed to encode (err=%d)",r);
+
+			av_packet_free(&pkt);
 		}
 	}
 }
 
 void ffmpeg_flushout() {
-	/* audio */
-	if (ffmpeg_fmt_ctx != NULL) {
-		if (ffmpeg_aud_frame != NULL) {
-			bool again;
-			AVPacket pkt;
-
-			if (ffmpeg_aud_write != 0)
-				ffmpeg_aud_frame->nb_samples = (int)ffmpeg_aud_write;
-
-			ffmpeg_audio_frame_send();
-			ffmpeg_aud_write = 0;
-			ffmpeg_aud_frame->nb_samples = 0;
-
-			do {
-				again=false;
-				av_init_packet(&pkt);
-				if (av_new_packet(&pkt,65536) == 0) {
-					int gotit=0;
-					if (avcodec_encode_audio2(ffmpeg_aud_ctx,&pkt,NULL,&gotit) == 0) {
-						if (gotit) {
-							again = true;
-							pkt.stream_index = ffmpeg_aud_stream->index;
-							av_packet_rescale_ts(&pkt,ffmpeg_aud_ctx->time_base,ffmpeg_aud_stream->time_base);
-
-							if (av_interleaved_write_frame(ffmpeg_fmt_ctx,&pkt) < 0)
-								LOG_MSG("WARNING: av_interleaved_write_frame failed");
-						}
-					}
-				}
-				av_packet_unref(&pkt);
-			} while (again);
-		}
-	}
-
+	ffmpeg_flush_audio();
 	ffmpeg_flush_video();
 }
 #endif
@@ -407,6 +415,7 @@ int ffmpeg_bpp_pick_rgb_format(int bpp) {
 void ffmpeg_reopen_video(double fps,const int bpp) {
 	if (ffmpeg_vid_ctx != NULL) {
 		avcodec_close(ffmpeg_vid_ctx);
+		avcodec_free_context(&ffmpeg_vid_ctx);
 		ffmpeg_vid_ctx = NULL;
 	}
 	if (ffmpeg_vidrgb_frame != NULL) {
@@ -425,9 +434,8 @@ void ffmpeg_reopen_video(double fps,const int bpp) {
 	LOG_MSG("Restarting video codec");
 
 	// FIXME: This is copypasta! Consolidate!
-	ffmpeg_vid_ctx = ffmpeg_vid_stream->codec = avcodec_alloc_context3(ffmpeg_vid_codec);
+	ffmpeg_vid_ctx = avcodec_alloc_context3(ffmpeg_vid_codec);
 	if (ffmpeg_vid_ctx == NULL) E_Exit("Error: Unable to reopen vid codec");
-	avcodec_get_context_defaults3(ffmpeg_vid_ctx,ffmpeg_vid_codec);
 	ffmpeg_vid_ctx->bit_rate = 25000000; // TODO: make configuration option!
 	ffmpeg_vid_ctx->keyint_min = 15; // TODO: make configuration option!
 	ffmpeg_vid_ctx->time_base.num = 1000000;
@@ -452,6 +460,8 @@ void ffmpeg_reopen_video(double fps,const int bpp) {
 	{
 		AVDictionary *opts = NULL;
 
+		av_dict_set(&opts,"crf","14",1); /* default is 20, this allows higher quality */
+		av_dict_set(&opts,"crf_max","20",1); /* don't let it get too low quality */
 		av_dict_set(&opts,"preset","superfast",1);
 		av_dict_set(&opts,"aud","1",1);
 
@@ -462,12 +472,16 @@ void ffmpeg_reopen_video(double fps,const int bpp) {
 		av_dict_free(&opts);
 	}
 
+	if (avcodec_parameters_from_context(ffmpeg_vid_stream->codecpar, ffmpeg_vid_ctx) < 0) {
+		E_Exit("failed to copy video codec parameters");
+	}
+
 	ffmpeg_vid_frame = av_frame_alloc();
 	ffmpeg_vidrgb_frame = av_frame_alloc();
 	if (ffmpeg_aud_frame == NULL || ffmpeg_vid_frame == NULL || ffmpeg_vidrgb_frame == NULL)
 		E_Exit(" ");
 
-	av_frame_set_colorspace(ffmpeg_vidrgb_frame,AVCOL_SPC_RGB);
+	ffmpeg_vidrgb_frame->colorspace = AVCOL_SPC_RGB;
 	ffmpeg_vidrgb_frame->width = (int)capture.video.width;
 	ffmpeg_vidrgb_frame->height = (int)capture.video.height;
 	ffmpeg_vidrgb_frame->format = ffmpeg_bpp_pick_rgb_format(bpp);
@@ -475,8 +489,8 @@ void ffmpeg_reopen_video(double fps,const int bpp) {
 		E_Exit(" ");
 	}
 
-	av_frame_set_colorspace(ffmpeg_vid_frame,AVCOL_SPC_SMPTE170M);
-	av_frame_set_color_range(ffmpeg_vidrgb_frame,AVCOL_RANGE_MPEG);
+	ffmpeg_vid_frame->colorspace = AVCOL_SPC_SMPTE170M;
+	ffmpeg_vidrgb_frame->color_range = AVCOL_RANGE_MPEG;
 	ffmpeg_vid_frame->width = (int)capture.video.width;
 	ffmpeg_vid_frame->height = (int)capture.video.height;
 	ffmpeg_vid_frame->format = ffmpeg_vid_ctx->pix_fmt;
@@ -1129,12 +1143,11 @@ skip_shot:
 				LOG_MSG("Failed to allocate format context (mpegts)");
 				goto skip_video;
 			}
-			snprintf(ffmpeg_fmt_ctx->filename,sizeof(ffmpeg_fmt_ctx->filename),"%s",path.c_str());
 
 			if (ffmpeg_fmt_ctx->oformat == NULL)
 				goto skip_video;
 
-			if (avio_open(&ffmpeg_fmt_ctx->pb,ffmpeg_fmt_ctx->filename,AVIO_FLAG_WRITE) < 0) {
+			if (avio_open(&ffmpeg_fmt_ctx->pb,pathvid.c_str(),AVIO_FLAG_WRITE) < 0) {
 				LOG_MSG("Failed to avio_open");
 				goto skip_video;
 			}
@@ -1144,8 +1157,8 @@ skip_shot:
 				LOG_MSG("failed to open audio stream");
 				goto skip_video;
 			}
-			ffmpeg_vid_ctx = ffmpeg_vid_stream->codec;
-			avcodec_get_context_defaults3(ffmpeg_vid_ctx,ffmpeg_vid_codec);
+			ffmpeg_vid_ctx = avcodec_alloc_context3(ffmpeg_vid_codec);
+			if (ffmpeg_vid_ctx == NULL) E_Exit("Error: Unable to open vid context");
 			ffmpeg_vid_ctx->bit_rate = 25000000; // TODO: make configuration option!
 			ffmpeg_vid_ctx->keyint_min = 15; // TODO: make configuration option!
 			ffmpeg_vid_ctx->time_base.num = 1000000;
@@ -1170,6 +1183,8 @@ skip_shot:
 			{
 				AVDictionary *opts = NULL;
 
+				av_dict_set(&opts,"crf","14",1); /* default is 20, this allows higher quality */
+				av_dict_set(&opts,"crf_max","20",1); /* don't let it get too low quality */
 				av_dict_set(&opts,"preset","superfast",1);
 				av_dict_set(&opts,"aud","1",1);
 
@@ -1184,19 +1199,29 @@ skip_shot:
 			ffmpeg_vid_stream->time_base.num = (int)1000000;
 			ffmpeg_vid_stream->time_base.den = (int)(1000000 * fps);
 
+			if (avcodec_parameters_from_context(ffmpeg_vid_stream->codecpar, ffmpeg_vid_ctx) < 0) {
+				LOG_MSG("failed to copy video codec parameters");
+				goto skip_video;
+			}
+
 			ffmpeg_aud_stream = avformat_new_stream(ffmpeg_fmt_ctx,ffmpeg_aud_codec);
 			if (ffmpeg_aud_stream == NULL) {
 				LOG_MSG("failed to open audio stream");
 				goto skip_video;
 			}
-			ffmpeg_aud_ctx = ffmpeg_aud_stream->codec;
-			avcodec_get_context_defaults3(ffmpeg_aud_ctx,ffmpeg_aud_codec);
+			ffmpeg_aud_ctx = avcodec_alloc_context3(ffmpeg_aud_codec);
+			if (ffmpeg_aud_ctx == NULL) E_Exit("Error: Unable to open aud context");
 			ffmpeg_aud_ctx->sample_rate = (int)capture.video.audiorate;
-			ffmpeg_aud_ctx->channels = 2;
 			ffmpeg_aud_ctx->flags = 0; // do not use global headers
 			ffmpeg_aud_ctx->bit_rate = 320000;
 			ffmpeg_aud_ctx->profile = FF_PROFILE_AAC_LOW;
+
+			#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(59,24,100)
+			ffmpeg_aud_ctx->channels = 2;
 			ffmpeg_aud_ctx->channel_layout = AV_CH_LAYOUT_STEREO;
+			#else
+			ffmpeg_aud_ctx->ch_layout = AV_CHANNEL_LAYOUT_STEREO;
+			#endif
 
 			if (ffmpeg_aud_codec->sample_fmts != NULL)
 				ffmpeg_aud_ctx->sample_fmt = (ffmpeg_aud_codec->sample_fmts)[0];
@@ -1211,15 +1236,20 @@ skip_shot:
 			ffmpeg_aud_stream->time_base.num = 1;
 			ffmpeg_aud_stream->time_base.den = ffmpeg_aud_ctx->sample_rate;
 
-			/* Note whether we started the header.
-			 * Writing the trailer out of turn seems to cause segfaults in libavformat */
-			ffmpeg_avformat_began = true;
+			if (avcodec_parameters_from_context(ffmpeg_aud_stream->codecpar, ffmpeg_aud_ctx) < 0) {
+				LOG_MSG("failed to copy audio codec parameters");
+				goto skip_video;
+			}
 
 			if (avformat_write_header(ffmpeg_fmt_ctx,NULL) < 0) {
 				LOG_MSG("Failed to write header");
 				goto skip_video;
 			}
 
+			/* Note whether we started the header.
+			 * Writing the trailer out of turn seems to cause segfaults in libavformat */
+			ffmpeg_avformat_began = true;
+
 			ffmpeg_aud_write = 0;
 			ffmpeg_aud_frame = av_frame_alloc();
 			ffmpeg_vid_frame = av_frame_alloc();
@@ -1227,9 +1257,14 @@ skip_shot:
 			if (ffmpeg_aud_frame == NULL || ffmpeg_vid_frame == NULL || ffmpeg_vidrgb_frame == NULL)
 				goto skip_video;
 
-			av_frame_set_channels(ffmpeg_aud_frame,2);
-			av_frame_set_sample_rate(ffmpeg_aud_frame,(int)capture.video.audiorate);
-			av_frame_set_channel_layout(ffmpeg_aud_frame,AV_CH_LAYOUT_STEREO);
+			#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(59,24,100)
+			ffmpeg_aud_frame->channels = 2;
+			ffmpeg_aud_frame->channel_layout = AV_CH_LAYOUT_STEREO;
+			#else
+			ffmpeg_aud_frame->ch_layout = AV_CHANNEL_LAYOUT_STEREO;
+			#endif
+
+			ffmpeg_aud_frame->sample_rate = (int)capture.video.audiorate;
 			ffmpeg_aud_frame->nb_samples = ffmpeg_aud_ctx->frame_size;
 			ffmpeg_aud_frame->format = ffmpeg_aud_ctx->sample_fmt;
 			if (av_frame_get_buffer(ffmpeg_aud_frame,16) < 0) {
@@ -1237,7 +1272,7 @@ skip_shot:
 				goto skip_video;
 			}
 
-			av_frame_set_colorspace(ffmpeg_vidrgb_frame,AVCOL_SPC_RGB);
+			ffmpeg_vidrgb_frame->colorspace = AVCOL_SPC_RGB;
 			ffmpeg_vidrgb_frame->width = (int)capture.video.width;
 			ffmpeg_vidrgb_frame->height = (int)capture.video.height;
 			ffmpeg_vidrgb_frame->format = ffmpeg_bpp_pick_rgb_format((int)bpp);
@@ -1246,8 +1281,8 @@ skip_shot:
 				goto skip_video;
 			}
 
-			av_frame_set_colorspace(ffmpeg_vid_frame,AVCOL_SPC_SMPTE170M);
-			av_frame_set_color_range(ffmpeg_vidrgb_frame,AVCOL_RANGE_MPEG);
+			ffmpeg_vid_frame->colorspace = AVCOL_SPC_SMPTE170M;
+			ffmpeg_vidrgb_frame->color_range = AVCOL_RANGE_MPEG;
 			ffmpeg_vid_frame->width = (int)capture.video.width;
 			ffmpeg_vid_frame->height = (int)capture.video.height;
 			ffmpeg_vid_frame->format = ffmpeg_vid_ctx->pix_fmt;
@@ -1357,11 +1392,12 @@ skip_shot:
 		}
 #if (C_AVCODEC)
 		else if (export_ffmpeg && ffmpeg_fmt_ctx != NULL) {
-			AVPacket pkt;
+			signed long long saved_dts;
+			AVPacket* pkt = av_packet_alloc();
+			int r;
 
-			// video
-			av_init_packet(&pkt);
-			if (av_new_packet(&pkt,50000000/8) == 0) {
+			if (!pkt) E_Exit("Error: Unable to alloc packet");
+			{
 				unsigned char *srcline,*dstline;
 				Bitu x;
 
@@ -1437,39 +1473,33 @@ skip_shot:
 					LOG_MSG("WARNING: sws_scale() failed");
 
 				// encode it
-				int gotit=0;
-				pkt.pts = (int64_t)capture.video.frames;
-				pkt.dts = (int64_t)capture.video.frames;
 				ffmpeg_vid_frame->pts = (int64_t)capture.video.frames; // or else libx264 complains about non-monotonic timestamps
 				ffmpeg_vid_frame->key_frame = ((capture.video.frames % 15) == 0)?1:0;
-				if (avcodec_encode_video2(ffmpeg_vid_ctx,&pkt,ffmpeg_vid_frame,&gotit) == 0) {
-					if (gotit) {
-						uint64_t tm;
 
-						tm = (uint64_t)pkt.pts;
-						pkt.stream_index = ffmpeg_vid_stream->index;
-						av_packet_rescale_ts(&pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
-						pkt.pts += (int64_t)ffmpeg_video_frame_time_offset;
-						pkt.dts += (int64_t)ffmpeg_video_frame_time_offset;
+				r=avcodec_send_frame(ffmpeg_vid_ctx,ffmpeg_vid_frame);
+				if (r < 0 && r != AVERROR(EAGAIN))
+					LOG_MSG("WARNING: avcodec_send_frame() video failed to encode (err=%d)",r);
 
-						if (av_interleaved_write_frame(ffmpeg_fmt_ctx,&pkt) < 0)
-							LOG_MSG("WARNING: av_interleaved_write_frame failed");
+				while ((r=avcodec_receive_packet(ffmpeg_vid_ctx,pkt)) >= 0) {
+					saved_dts = pkt->dts;
+					pkt->stream_index = ffmpeg_vid_stream->index;
+					av_packet_rescale_ts(pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
+					pkt->pts += (int64_t)ffmpeg_video_frame_time_offset;
+					pkt->dts += (int64_t)ffmpeg_video_frame_time_offset;
 
-						pkt.pts = (int64_t)tm + (int64_t)1;
-						pkt.dts = (int64_t)tm + (int64_t)1;
-						av_packet_rescale_ts(&pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
-						ffmpeg_video_frame_last_time = (uint64_t)pkt.pts;
-					}
-					else {
-						LOG_MSG("DEBUG: avcodec_encode_video2 delayed frame");
-						/* delayed frame */
-					}
-				}
-				else {
-					LOG_MSG("WARNING: avcodec_encode_video2() failed");
+					if (av_interleaved_write_frame(ffmpeg_fmt_ctx,pkt) < 0)
+						LOG_MSG("WARNING: av_interleaved_write_frame failed");
+
+					pkt->pts = (int64_t)saved_dts + (int64_t)1;
+					pkt->dts = (int64_t)saved_dts + (int64_t)1;
+					av_packet_rescale_ts(pkt,ffmpeg_vid_ctx->time_base,ffmpeg_vid_stream->time_base);
+					ffmpeg_video_frame_last_time = (uint64_t)pkt->pts;
 				}
+
+				if (r != AVERROR(EAGAIN))
+					LOG_MSG("WARNING: avcodec_receive_packet() video failed to encode (err=%d)",r);
 			}
-			av_packet_unref(&pkt);
+			av_packet_free(&pkt);
 			capture.video.frames++;
 
 			if ( capture.video.audioused ) {
@@ -1969,6 +1999,8 @@ void CAPTURE_Init() {
 	capturedir = proppath->realpath;
     SetGameState_Run(section->Get_int("saveslot")-1);
     noremark_save_state = !section->Get_bool("saveremark");
+    video_debug_overlay = section->Get_bool("video debug at startup");
+    mainMenu.get_item("video_debug_overlay").check(video_debug_overlay).refresh_item(mainMenu);
     mainMenu.get_item("noremark_savestate").check(noremark_save_state).refresh_item(mainMenu);
     force_load_state = section->Get_bool("forceloadstate");
     mainMenu.get_item("force_loadstate").check(force_load_state).refresh_item(mainMenu);
