@@ -1308,7 +1308,8 @@ bool CPrinter::processCommandChar(uint8_t ch)
 		    }
 		    curX = leftMargin;
 		    curY += lineSpacing;
-		    if (curY > bottomMargin)
+		    //LOG_MSG("curY:%f, bottomMargin:%f",curY, bottomMargin);
+		    if (curY > bottomMargin-0.0001) // goto the next page if curY is larger or slightly smaller than bottomMargin
 			    newPage(true, false);
 		    return true;
 	    case 0x0e:		//Select double-width printing (one line) (SO)
@@ -2224,7 +2225,7 @@ void CPrinter::outputPage()
 			}
 			else
 			{
-				// Find end of heterogenous area
+				// Find end of heterogeneous area
 				uint8_t diffCount = 1;
 				while (
                     diffCount < 128 && diffCount + pix < numpix && 
@@ -2322,7 +2323,7 @@ void CPrinter::fprintASCII85(FILE* f, uint16_t b)
 	}
 	else // Close string
 	{
-		// Partial tupel if there are still bytes in the buffer
+		// Partial tuple if there are still bytes in the buffer
 		if (ASCII85BufferPos > 0)
 		{
 			for (uint8_t i = ASCII85BufferPos; i < 4; i++)
