@@ -176,9 +176,10 @@ unsigned char*                      pc98_pgraph_current_display_page;
 unsigned char*                      pc98_pgraph_current_cpu_page;
 
 bool                                vga_8bit_dac = false;
-bool                                vga_alt_new_mode = false;
 bool                                enable_vga_8bit_dac = true;
 bool                                ignore_sequencer_blanking = false;
+bool                                memio_complexity_optimization = true;
+bool                                vga_render_on_demand = false; // Render at vsync or specific changes to hardware instead of every scanline
 
 bool                                pc98_crt_mode = false;      // see port 6Ah command 40h/41h.
                                                                 // this boolean is the INVERSE of the bit.
@@ -447,8 +448,12 @@ void VGA_SetClock(Bitu which,Bitu target) {
     VGA_StartResize();
 }
 
+uint8_t CGAPal2[2] = {0,0};
+uint8_t CGAPal4[4] = {0,0,0,0};
+
 void VGA_SetCGA2Table(uint8_t val0,uint8_t val1) {
     const uint8_t total[2] = {val0,val1};
+    for (unsigned int i=0;i < 2;i++) CGAPal2[i] = total[i];
     for (Bitu i=0;i<16u;i++) {
         CGA_2_Table[i]=
 #ifdef WORDS_BIGENDIAN
@@ -468,6 +473,7 @@ void VGA_SetCGA2Table(uint8_t val0,uint8_t val1) {
 
 void VGA_SetCGA4Table(uint8_t val0,uint8_t val1,uint8_t val2,uint8_t val3) {
     const uint8_t total[4] = {val0,val1,val2,val3};
+    for (unsigned int i=0;i < 4;i++) CGAPal4[i] = total[i];
     for (Bitu i=0;i<256u;i++) {
         CGA_4_Table[i]=
 #ifdef WORDS_BIGENDIAN
@@ -926,6 +932,16 @@ void VGA_Reset(Section*) {
     vga_8bit_dac = false;
     enable_vga_8bit_dac = section->Get_bool("enable 8-bit dac");
     ignore_sequencer_blanking = section->Get_bool("ignore sequencer blanking");
+    memio_complexity_optimization = section->Get_bool("memory io optimization 1");
+    vga_render_on_demand = section->Get_bool("scanline render on demand");
+
+    if (memio_complexity_optimization)
+        LOG_MSG("Memory I/O complexity optimization enabled aka option 'memory io optimization 1'. If the game or demo is unable to draw to the screen properly, set the option to false.");
+
+    if (vga_render_on_demand)
+        LOG_MSG("'scanline render on demand' option is enabled. If this option breaks the game or demo effects or display, set the option to false.");
+    else
+        LOG_MSG("The 'scanline render on demand' option is available and may provide a modest boost in video render performance if set to true.");
 
     vga_memio_delay_ns = section->Get_int("vmemdelay");
     if (vga_memio_delay_ns < 0) {
@@ -1038,7 +1054,7 @@ void VGA_Reset(Section*) {
             // TODO: There are reports of VGA cards that have less than 256KB in the early days of VGA.
             //       How does that work exactly, especially when 640x480 requires about 37KB per plane?
             //       Did these cards have some means to chain two bitplanes odd/even in the same way
-            //       tha EGA did it?
+            //       than EGA did it?
             if (vga.mem.memsize != 0 || svgaCard == SVGA_None) {
                 if (vga.mem.memsize < _KB_bytes(256)) vga.mem.memsize = _KB_bytes(256);
             }
@@ -1314,7 +1330,15 @@ static const UINT8 gdc_defsyncm31[8] = {0x10,0x4e,0x47,0x0c,0x07,0x0d,0x90,0x89}
 static const UINT8 gdc_defsyncs31[8] = {0x06,0x26,0x41,0x0c,0x83,0x0d,0x90,0x89};
 
 static const UINT8 gdc_defsyncm31_480[8] = {0x10,0x4e,0x4b,0x0c,0x03,0x06,0xe0,0x95};
-static const UINT8 gdc_defsyncs31_480[8] = {0x06,0x4e,0x4b,0x0c,0x83,0x06,0xe0,0x95};
+static const UINT8 gdc_defsyncs31_480[8] = {0x06,0x26,0x41,0x0c,0x83,0x06,0xe0,0x95};
+
+/* ^ NTS: Even in 256-color mode the expectation for Active Display Words is 40, not 80.
+ *        Writing 80 when INT 18h is used to invoke 256-color mode isn't a problem but
+ *        it does cause erroneous values in debug functions (1280x480 for
+ *        "login 256-color paint tool"?). The correct value is 40 according to NP2
+ *        source code (in what cases exactly?) and some games that manually set up
+ *        640x400 16-color mode AND THEN directly switch to 256-color mode (still 640x400)
+ *        expect the ADW to remain 40 ("battle skin panic"). */
 
 void PC98_Set24KHz(void) {
     pc98_gdc[GDC_MASTER].write_fifo_command(0x0F/*sync DE=1*/);
@@ -1747,17 +1771,17 @@ void JEGA_setupAX(void) {
 	jega.RDFFB = 0x00;//bc: Font access first byte
 	jega.RDFSB = 0x00;//bd: second
 	jega.RDFAP = 0x00;//be: Font Access Pattern
-	jega.RPESL = 0x00;//09: end scan line (superceded by EGA)
-	jega.RPULP = 0x00;//14: under scan line (superceded by EGA)
+	jega.RPESL = 0x00;//09: end scan line (superseded by EGA)
+	jega.RPULP = 0x00;//14: under scan line (superseded by EGA)
 	jega.RPSSC = 1;//db: DBCS start scan line
 	jega.RPSSU = 3;//d9: 2x DBCS upper start scan
 	jega.RPSSL = 0;//da: 2x DBCS lower start scan
 	jega.RPPAJ = 0x00;//dc: super imposed (only AX-2 system, not implemented)
 	jega.RCMOD = 0x00;//dd: Cursor Mode (not implemented)
-	jega.RCCLH = 0x00;//0e: Cursor location Upper bits (superceded by EGA)
-	jega.RCCLL = 0x00;//0f: Cursor location Lower bits (superceded by EGA)
-	jega.RCCSL = 0x00;//0a: Cursor Start Line (superceded by EGA)
-	jega.RCCEL = 0x00;//0b: Cursor End Line (superceded by EGA)
+	jega.RCCLH = 0x00;//0e: Cursor location Upper bits (superseded by EGA)
+	jega.RCCLL = 0x00;//0f: Cursor location Lower bits (superseded by EGA)
+	jega.RCCSL = 0x00;//0a: Cursor Start Line (superseded by EGA)
+	jega.RCCEL = 0x00;//0b: Cursor End Line (superseded by EGA)
 	jega.RCSKW = 0x20;//de: Cursor Skew control (not implemented)
 	jega.ROMSL = 0x00;//df: Unused?
 	jega.RSTAT = 0x03;//bf: Font register accessible status
