@@ -25,8 +25,19 @@
 /* do not issue CPU-side I/O here -- this code emulates functions that the GDC itself carries out, not on the CPU */
 #include "cpu_io_is_forbidden.h"
 
+extern bool dbg_event_color_plane_enable;
+extern bool dbg_event_color_select;
+extern bool vga_render_on_demand;
+void VGA_RenderOnDemandUpTo(void);
+
 #define attr(blah) vga.attr.blah
 
+static EGAMonitorMode currentMonitorMode = CGA;
+
+EGAMonitorMode egaMonitorMode(void) {
+	return currentMonitorMode;
+}
+
 void VGA_ATTR_SetEGAMonitorPalette(EGAMonitorMode m) {
 	// palette bit assignment:
 	// bit | pin | EGA        | CGA       | monochrome
@@ -39,6 +50,7 @@ void VGA_ATTR_SetEGAMonitorPalette(EGAMonitorMode m) {
 	// 5   | 2   | red sec.   | nc        | nc
     // 6-7 | not used
 	// * additive color brown instead of yellow
+	currentMonitorMode = m;
 	switch(m) {
 		case CGA:
 			//LOG_MSG("Monitor CGA");
@@ -135,6 +147,19 @@ bool J3_IsCga4Dcga();
 
 void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
 	if (!vga.internal.attrindex) {
+		/* Render on Demand problem:
+		 *
+		 *          If we act on all writes as a cue to render a scanline, what will likely happen is the
+		 *          scanline will be rendered blank. The reason is that when writing the palette in the
+		 *          Attribute Controller you are supposed to set bit 0x20 to disable the screen while you
+		 *          do so. Acting upon all writes will likely lead to a render while screen is disabled.
+		 *          Therefore, only trigger when the screen is enabled.
+		 *
+		 *          This means that render on demand will probably not 100% render things correctly for
+		 *          very timing dependent tricks, but will render most DOS games with better performance
+		 *          which for most casual users of DOSBox and DOSBox-X is just fine. Just don't expect
+		 *          this performance enhancement to handle all the tricks of the Demoscene. --J.C. */
+		if (vga_render_on_demand && !attr(disabled)/*screen not yet disabled*/) VGA_RenderOnDemandUpTo();
 		attr(index)=val & 0x1F;
 		vga.internal.attrindex=true;
 		if (val & 0x20) attr(disabled) &= ~1;
@@ -157,6 +182,7 @@ void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
          */
 		return;
 	} else {
+		if (vga_render_on_demand && !attr(disabled)/*screen not disabled*/) VGA_RenderOnDemandUpTo();
 		vga.internal.attrindex=false;
 		switch (attr(index)) {
 			/* Palette */
@@ -247,6 +273,7 @@ void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
 			/* Why disable colour planes? */
 			/* To support weird modes. */
 			if ((attr(color_plane_enable)^val) & 0xf) {
+				dbg_event_color_plane_enable = true;
 				// in case the plane enable bits change...
 				attr(color_plane_enable)=(uint8_t)val;
 				for (uint8_t i=0;i<0x10;i++)
@@ -302,6 +329,7 @@ void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
 				break;
 			}
 			if (attr(color_select) ^ val) {
+				dbg_event_color_select = true;
 				attr(color_select)=(uint8_t)val;
 				for (uint8_t i=0;i<0x10;i++)
 					VGA_ATTR_SetPalette(i,vga.attr.palette[i]);
@@ -319,7 +347,7 @@ void write_p3c0(Bitu /*port*/,Bitu val,Bitu iolen) {
 				svga.write_p3c0(attr(index), val, iolen);
 				break;
 			}
-			LOG(LOG_VGAMISC,LOG_NORMAL)("VGA:ATTR:Write to unkown Index %2X",attr(index));
+			LOG(LOG_VGAMISC,LOG_NORMAL)("VGA:ATTR:Write to unknown Index %2X",attr(index));
 			break;
 		}
 	}
@@ -347,7 +375,7 @@ Bitu read_p3c1(Bitu /*port*/,Bitu iolen) {
 	default:
 		if (svga.read_p3c1)
 			return svga.read_p3c1(attr(index), iolen);
-		LOG(LOG_VGAMISC,LOG_NORMAL)("VGA:ATTR:Read from unkown Index %2X",attr(index));
+		LOG(LOG_VGAMISC,LOG_NORMAL)("VGA:ATTR:Read from unknown Index %2X",attr(index));
 	}
 	return 0;
 }
