@@ -813,7 +813,7 @@ static void write_tandy_reg(uint8_t val) {
 		}
 		break;
 	case 0x1:	/* Palette mask */
-		vga.tandy.palette_mask = val;
+		vga.tandy.palette_mask = val; // FIXME: Wait... does this apply always in every mode or just the CGA compatible modes?
 		tandy_update_palette();
 		break;
 	case 0x2:	/* Border color */
@@ -877,7 +877,7 @@ static void write_tandy(Bitu port,Bitu val,Bitu /*iolen*/) {
 		// The remapped range is 32kB instead of 16. Therefore CPU_PG bit 0
 		// appears to be ORed with CPU A14 (to preserve some sort of
 		// backwards compatibility?), resulting in odd pages being mapped
-		// as 2x16kB. Implemeted in vga_memory.cpp Tandy handler.
+		// as 2x16kB. Implemented in vga_memory.cpp Tandy handler.
 
 		vga.tandy.line_mask = (uint8_t)(val >> 6);
 		vga.tandy.draw_bank = val & ((vga.tandy.line_mask&2) ? 0x6 : 0x7);
