@@ -16,6 +16,7 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <queue>
 
 #include "dosbox.h"
 #include "callback.h"
@@ -295,7 +296,6 @@ static scancode_tbl scan_to_scanascii_pc98[0x80] = {
     {   none,   none,   none,   none,   none,   none }  /* 7f      */
 };
 
-#include <queue>
 std::queue <uint16_t>over_key_buffer;
 
 extern bool inshell;
@@ -363,7 +363,7 @@ bool BIOS_AddKeyToBuffer(uint16_t code) {
         }
     }
     /* Check for buffer Full */
-    //TODO Maybe beeeeeeep or something although that should happend when internal buffer is full
+    //TODO Maybe beeeeeeep or something although that should happened when internal buffer is full
     if (ttail==head) {
 #if defined(USE_TTF)
         if(IS_DOSV || ttf_dosv) {
@@ -722,7 +722,6 @@ static Bitu IRQ1_Handler(void) {
     case 0x53: /* del . Not entirely correct, but works fine */
         if (scancode == 0x53 && !(flags3 & 0x01) && !(flags1 & 0x03) && (flags1 & 0x0c) == 0x0c && ((!(flags3 & 0x10) && (flags3 & 0x0c) == 0x0c) || ((flags3 & 0x10) && (flags2 & 0x03) == 0x03))) { /* Ctrl-Alt-Del? */
 			throw int(3);
-            break;
 		}
         if(flags3 &0x02) {  /*extend key. e.g key above arrows or arrows*/
             if(scancode == 0x52) flags2 |=0x80; /* press insert */         
@@ -1080,7 +1079,6 @@ static Bitu IRQ1_Handler_PC98(void) {
                             add_key(scan_add + '\'');
                             break;
                         }
-                        break;
                     default:
                         if (shift){
                             if (scan_to_scanascii_pc98[sc_8251].shift) add_key(scan_to_scanascii_pc98[sc_8251].shift);
@@ -1162,9 +1160,20 @@ static bool IsEnhancedKey(uint16_t &key) {
     return false;
 }
 
-#if defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+#if defined(C_SDL2)
+#if defined(WIN32) && !defined(HX_DOS)
 extern void IME_SetEnable(BOOL state);
 extern bool IME_GetEnable();
+#elif defined(MACOSX)
+extern bool IME_GetEnable();
+extern void IME_SetEnable(int state);
+#ifndef TRUE
+#define TRUE 1
+#endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+#endif
 #endif
 
 extern bool DOS_BreakFlag;
@@ -1296,7 +1305,7 @@ Bitu INT16_Handler(void) {
                  (mem_readb(BIOS_KEYBOARD_FLAGS3)&0x0c);    // Right Ctrl/Alt pressed, bits 2,3
         break;
     case 0x13:
-#if defined(WIN32) && !defined(HX_DOS) && !defined(C_SDL2) && defined(SDL_DOSBOX_X_SPECIAL)
+#if (defined(WIN32) && !defined(HX_DOS) || defined(MACOSX)) && !defined(C_SDL2) && defined(SDL_DOSBOX_X_SPECIAL)
 #if defined(USE_TTF)
         if((IS_DOSV || ttf_dosv) && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
 #else
@@ -1316,7 +1325,7 @@ Bitu INT16_Handler(void) {
                 }
             }
         }
-#elif defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+#elif (defined(WIN32) && !defined(HX_DOS) || defined(MACOSX)) && defined(C_SDL2)
 #if defined(USE_TTF)
         if((IS_DOSV || ttf_dosv) && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
 #else
