@@ -389,7 +389,26 @@ Bitu PS2_Handler(void) {
 #define MOUSE_WHEEL_MOVED 128
 #define MOUSE_ABSOLUTE 256
 #define MOUSE_DUMMY 256
-#define MOUSE_DELAY 5.0
+
+unsigned int user_mouse_report_rate = 0;
+unsigned int mouse_report_rate = 200; /* DOSBox SVN compatible default (MOUSE_DELAY = 5.0 ms) */
+pic_tickindex_t MOUSE_DELAY = 5.0; /* This was once a hard #define */
+
+void UpdateMouseReportRate(void) {
+	/* If the user did not specify an explicit rate, then whatever the current rate goes and update MOUSE_DELAY.
+	 * This is done so that PS/2 mouse emulation can accept the commands to change reporting rate and have it work */
+	if (user_mouse_report_rate != 0)
+		mouse_report_rate = user_mouse_report_rate;
+
+	MOUSE_DELAY = 1000.0 / mouse_report_rate;
+}
+
+void ChangeMouseReportRate(unsigned int new_rate) {
+	if (user_mouse_report_rate == 0) {
+		mouse_report_rate = new_rate;
+		UpdateMouseReportRate();
+	}
+}
 
 void MOUSE_Limit_Events(Bitu /*val*/) {
     mouse.timer_in_progress = false;
@@ -1023,12 +1042,12 @@ void Mouse_Select(int x1, int y1, int x2, int y2, int w, int h, bool select) {
 #if defined(USE_TTF)
     ttfuse = ttf.inUse;
     ttfcols = ttf.cols;
-    if (ttfuse&&(!IS_EGAVGA_ARCH||CurMode->mode!=3||isDBCSCP()&&dbcs_sbcs)) {
+    if (ttfuse&&(!IS_EGAVGA_ARCH||CurMode->mode!=3||(isDBCSCP()&&dbcs_sbcs))) {
         ttf_cell *newAC = newAttrChar;
         for (unsigned int y = 0; y < ttf.lins; y++) {
             if (y>=r1&&y<=r2)
                 for (unsigned int x = 0; x < ttf.cols; x++)
-                    if ((x>=c1||((IS_PC98_ARCH||isDBCSCP()&&dbcs_sbcs)&&c1>0&&x==c1-1&&(newAC[rtl?ttf.cols-x-1:x].chr&0xFF00)&&(newAC[rtl?ttf.cols-x:x+1].chr&0xFF)==32))&&x<=c2)
+                    if ((x>=c1||((IS_PC98_ARCH||(isDBCSCP()&&dbcs_sbcs))&&c1>0&&x==c1-1&&(newAC[rtl?ttf.cols-x-1:x].chr&0xFF00)&&(newAC[rtl?ttf.cols-x:x+1].chr&0xFF)==32))&&x<=c2)
                         newAC[rtl?ttf.cols-x-1:x].selected = select?1:0;
             newAC += ttf.cols;
         }
@@ -1771,7 +1790,7 @@ static Bitu INT33_Handler(void) {
              *                        04h     Swedish
              *                        05h     Finnish
              *                        06h     Spanish
-             *                        07h     Portugese
+             *                        07h     Portuguese
              *                        08h     Italian
              *
              */
@@ -2116,6 +2135,9 @@ void MOUSE_Startup(Section *sec) {
         return;
     }
 
+    user_mouse_report_rate=section->Get_int("mouse report rate");
+    UpdateMouseReportRate();
+
     en_int33_hide_if_intsub=section->Get_bool("int33 hide host cursor if interrupt subroutine");
 
     en_int33_hide_if_polling=section->Get_bool("int33 hide host cursor when polling");
@@ -2296,6 +2318,8 @@ static inline void CmdAbsPointerCommand() {
 // IO port handling
 
 static Bitu PortRead(Bitu port, Bitu iolen) {
+    (void)port;
+    (void)iolen;
 
         if (reg_eax != VMWARE_MAGIC)
                 return 0;
