@@ -246,7 +246,7 @@ void Program::WriteOut(const char * format,...) {
 }
 
 void Program::WriteOut(const char *format, const char *arguments) {
-	char buf[2048];
+	char buf[2048 + CMD_MAXLINE];
 	sprintf(buf,format,arguments);
 
 	uint16_t size = (uint16_t)strlen(buf);
@@ -722,6 +722,11 @@ void ApplySetting(std::string pvar, std::string inputline, bool quiet) {
                     sdl.mouse.xsensitivity = p3->GetSection()->Get_int("xsens");
                     sdl.mouse.ysensitivity = p3->GetSection()->Get_int("ysens");
                 }
+#if C_GAMELINK
+                if (!strcasecmp(inputline.substr(0, 22).c_str(), "gamelink load address=")) {
+                    sdl.gamelink.loadaddr = section->Get_int("gamelink load address");
+                }
+#endif
                 if (!strcasecmp(inputline.substr(0, 11).c_str(), "fullscreen=")) {
                     if (section->Get_bool("fullscreen")) {
                         if (!GFX_IsFullscreen()) {GFX_LosingFocus();GFX_SwitchFullScreen();}
